SystemOrganization addCategory: #'Pier-TagCloud'!

PRWidget subclass: #PRTagCloudWidget
	instanceVariableNames: 'tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-TagCloud'!

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 9/13/2007 09:31'!
descriptionMaxItems
	^ MANumberDescription new
		label: 'How many tags to show?';
		selectorAccessor: #maxItems;
		priority: 210;
		default: 50;
		beInteger;
		min: 1;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 9/13/2007 09:32'!
descriptionShowFrequencies
	^ MABooleanDescription new
		selectorAccessor: #showFrequencies;
		label: 'Show frequencies?';
		default: false;
		priority: 220;
		yourself! !

!PRTagCloudWidget class methodsFor: 'accessing-description' stamp: 'lr 9/12/2007 22:22'!
descriptionStructure
	^ PRStructureDescription new
		selectorAccessor: #startAt;
		label: 'Start At';
		priority: 200;
		yourself! !

!PRTagCloudWidget class methodsFor: 'testing' stamp: 'lr 9/12/2007 22:55'!
isAbstract
	^ false! !

!PRTagCloudWidget class methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:54'!
label
	^ 'Tag Cloud'! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:32'!
enumerator
	"Answer an enumerator to visit the necessary structures to collect the tags."
	
	^ (self context enumeratorFor: self structure) with all! !

!PRTagCloudWidget methodsFor: 'events' stamp: 'lr 9/12/2007 22:55'!
onChangeContext: aContext
	super onChangeContext: aContext.
	tags := nil! !

!PRTagCloudWidget methodsFor: 'rendering' stamp: 'lr 9/13/2007 09:36'!
renderContentOn: html
	| max |
	self tags isEmpty ifTrue: [ ^ self ].
	max := self tagsByCount last value size.
	max > 1 ifTrue: [ max := max - 1 ].
	self tagsByName 
		do: [ :assoc |
			html span class: 'tag'; with: [
				html anchor
					callback: [ self show: assoc value ];
					style: 'font-size: ' , (assoc value size - 1 * 100 / max + 100)
						truncated asString , '%';
					with: assoc key.
				(self read: #descriptionShowFrequencies) ifTrue: [
					html span class: 'frequency'; with: [
						html text: '('; render: assoc value size; text: ')' ] ] ] ]
		separatedBy: [ html text: ' ' ]! !

!PRTagCloudWidget methodsFor: 'actions' stamp: 'lr 9/12/2007 23:18'!
show: aCollection
	aCollection size = 1
		ifTrue: [ ^ self context: (self context structure: aCollection atRandom) ].
	self inform: [ :html |
		html unorderedList: [
			aCollection do: [ :each |
				html listItem: [
					html anchor
						callback: [ 
							self context: (self context structure: each).
							self home ];
						with: each title ] ] ] ]! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:23'!
structure
	"Answer the structure where the search for the tag cloud should start from."
	
	^ (self read: #descriptionStructure) ifNil: [ self context structure ]! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 23:17'!
tags
	"Answer a dictionary mapping tags to structures."
	
	tags isNil ifFalse: [ ^ tags ].
	tags := Dictionary new.
	self enumerator do: [ :struct |
		struct tags do: [ :each |
			(tags at: each asLowercase ifAbsentPut: [ Set new ])
				add: struct ] ].
	^ tags! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:36'!
tagsByCount
	"Answer the tags sorted by count: smallest first, biggest last."
	
	| result |
	result := SortedCollection sortBlock: [ :a :b | a value size < b value size ].
	self tags associationsDo: [ :assoc | result add: assoc ].
	^ result! !

!PRTagCloudWidget methodsFor: 'accessing' stamp: 'lr 9/12/2007 22:36'!
tagsByName
	"Answer the tags alphabetically sorted by name."
	
	| result |
	result := SortedCollection sortBlock: [ :a :b | a key < b key ].
	self tags associationsDo: [ :assoc | result add: assoc ].
	^ result! !
